@@ -1,5 +1,5 @@
-#! /usr/bin/env python
-"""Jpylyzer validator for JPEG 200 Part 1 (JP2) images.
+#! /usr/bin/env python3tilePart
+"""Jpylyzer validator for JPEG 2000 Part 1 (JP2) images.
 
 Requires: Python 2.7 (older versions won't work) OR Python 3.2 or more recent
   (Python 3.0 and 3.1 won't work either!)
@@ -17,6 +17,7 @@ Contributors:
    Stefan Weil, UB Mannheim, DE.
    Adam Fritzler, Planet Labs, USA.
    Thomas Ledoux, Bibliotheque Nationale de France
+   Tim Lander, Hexagon Geospatial
 """
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU Lesser General Public License as published by
@@ -32,7 +33,6 @@ Contributors:
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
 import sys
-import mmap
 import os
 import time
 import datetime
@@ -42,14 +42,22 @@ import codecs
 import re
 from xml.dom import minidom
 import xml.etree.ElementTree as ETree
-if sys.version.startswith("2"):
-    from six import u
 from . import config
 from . import etpatch as ET
 from . import boxvalidator as bv
 from . import mix
 from . import shared
-
+try:
+    import mmap
+    NO_MMAP_LIB = False
+except ImportError:
+    NO_MMAP_LIB = True
+if sys.version.startswith("2"):
+    try:
+        from six import u
+        NO_U_LIB = False
+    except ImportError:
+        NO_U_LIB = True
 
 SCRIPT_PATH, SCRIPT_NAME = os.path.split(sys.argv[0])
 
@@ -57,7 +65,7 @@ SCRIPT_PATH, SCRIPT_NAME = os.path.split(sys.argv[0])
 if not SCRIPT_NAME:
     SCRIPT_NAME = 'jpylyzer'
 
-__version__ = "2.0.1"
+__version__ = "2.1.0"
 
 # Create PARSER
 PARSER = argparse.ArgumentParser(
@@ -73,7 +81,8 @@ XSI_NS_STRING = 'http://www.w3.org/2001/XMLSchema-instance'
 LOC_SCHEMA_STRING_1 = 'http://openpreservation.org/ns/jpylyzer/ \
 http://jpylyzer.openpreservation.org/jpylyzer-v-1-1.xsd'
 LOC_SCHEMA_STRING_2 = 'http://openpreservation.org/ns/jpylyzer/v2/ \
-http://jpylyzer.openpreservation.org/jpylyzer-v-2-0.xsd'
+http://jpylyzer.openpreservation.org/jpylyzer-v-2-1.xsd'
+
 
 def generatePropertiesRemapTable():
     """Generate nested dictionary.
@@ -260,28 +269,48 @@ def generatePropertiesRemapTable():
     return enumerationsMap
 
 
+def fileToBytes(filename):
+    """Read file, return contents as a byte object."""
+    fileData = ""
+    # Open file
+    with open(filename, "rb") as f:
+        fileData = f.read()
+
+    return fileData
+
+
 def fileToMemoryMap(filename):
     """Read contents of filename to memory map object."""
-    # Open filename
-    f = open(filename, "rb")
-
     # Call to mmap is different on Linux and Windows, so we need to know
     # the current platform
     platform = config.PLATFORM
 
-    try:
-        if platform == "win32":
-            # Parameters for Windows may need further fine-tuning ...
-            fileData = mmap.mmap(f.fileno(), 0, access=mmap.ACCESS_READ)
-        else:
-            # This works for Linux (and Cygwin on Windows). Not too sure
-            # about other platforms like Mac OS though
-            fileData = mmap.mmap(f.fileno(), 0, mmap.MAP_SHARED, mmap.PROT_READ)
-    except ValueError:
-        # mmap fails on empty files.
-        fileData = ""
+    # Open filename
+    with open(filename, "rb") as f:
+        try:
+            if platform == "win32":
+                # Parameters for Windows may need further fine-tuning ...
+                fileData = mmap.mmap(f.fileno(), 0, access=mmap.ACCESS_READ)
+            else:
+                # This works for Linux (and Cygwin on Windows). Not too sure
+                # about other platforms like Mac OS though
+                fileData = mmap.mmap(f.fileno(), 0, mmap.MAP_SHARED, mmap.PROT_READ)
+        except ValueError as e:
+            fileSize = os.path.getsize(filename)
+            if fileSize > sys.maxsize:
+                # Report warning if file size exceeds system maximum size
+                # on 32-bit Python
+                msg = ("file " + filename + " too large to open (" +
+                       str(round(fileSize/1024**3, 1)) +
+                       " GB). Try using 64-bit Python.")
+                shared.printWarning(msg)
+                fileData = ""
+            elif str(e) == "cannot mmap an empty file":
+                # mmap fails on empty files, so return empty string
+                msg = "empty file " + filename
+                shared.printWarning(msg)
+                fileData = ""
 
-    f.close()
     return fileData
 
 
@@ -319,10 +348,6 @@ def checkOneFile(path, validationFormat='jp2'):
     fileName = os.path.basename(path)
     filePath = os.path.abspath(path)
 
-    ## TEST
-    #print(fileName)
-    ## TEST
-
     # If file name / path contain any surrogate pairs, remove them to
     # avoid problems when writing to XML
     fileNameCleaned = stripSurrogatePairs(fileName)
@@ -352,7 +377,10 @@ def checkOneFile(path, validationFormat='jp2'):
 
     try:
         # Contents of file to memory map object
-        fileData = fileToMemoryMap(path)
+        if NO_MMAP_LIB:
+            fileData = fileToBytes(path)
+        else:
+            fileData = fileToMemoryMap(path)
 
         # Validate according to value of validation format
         if validationFormat == 'jp2':
@@ -364,7 +392,7 @@ def checkOneFile(path, validationFormat='jp2'):
         tests = resultsJP2.tests
         characteristics = resultsJP2.characteristics
 
-        if fileData != "":
+        if not NO_MMAP_LIB and fileData != "":
             fileData.close()
 
         # Generate property values remap table
@@ -430,6 +458,7 @@ def checkOneFile(path, validationFormat='jp2'):
 
     return root
 
+
 def checkNullArgs(args):
     """Check if the passed args list.
 
@@ -476,19 +505,32 @@ def stripSurrogatePairs(ustring):
 
     if config.PYTHON_VERSION.startswith(config.PYTHON_2):
         # Generate regex for surrogate pair detection
-
-        lone = re.compile(
-            u(r"""(?x)            # verbose expression (allows comments)
-            (                    # begin group
-            [\ud800-\udbff]      #   match leading surrogate
-            (?![\udc00-\udfff])  #   but only if not followed by trailing surrogate
-            )                    # end group
-            |                    #  OR
-            (                    # begin group
-            (?<![\ud800-\udbff]) #   if not preceded by leading surrogate
-            [\udc00-\udfff]      #   match trailing surrogate
-            )                   # end group
-            """))
+        if NO_U_LIB:
+            lone = re.compile(
+                r"""(?x)             # verbose expression (allows comments)
+                (                    # begin group
+                [\ud800-\udbff]      #   match leading surrogate
+                (?![\udc00-\udfff])  #   but only if not followed by trailing surrogate
+                )                    # end group
+                |                    #  OR
+                (                    # begin group
+                (?<![\ud800-\udbff]) #   if not preceded by leading surrogate
+                [\udc00-\udfff]      #   match trailing surrogate
+                )                    # end group
+                """)
+        else:
+            lone = re.compile(
+                u(r"""(?x)           # verbose expression (allows comments)
+                (                    # begin group
+                [\ud800-\udbff]      #   match leading surrogate
+                (?![\udc00-\udfff])  #   but only if not followed by trailing surrogate
+                )                    # end group
+                |                    #  OR
+                (                    # begin group
+                (?<![\ud800-\udbff]) #   if not preceded by leading surrogate
+                [\udc00-\udfff]      #   match trailing surrogate
+                )                    # end group
+                """))
 
         # Remove surrogates (i.e. replace by empty string)
         tmp = lone.sub(r'', ustring).encode('utf-8')
@@ -548,7 +590,7 @@ def findFiles(recurse, paths):
             root = unicode(root, 'utf-8')
 
         # WILDCARD IN PATH OR FILENAME
-        # In Linux wilcard expansion done by bash so, add file to list
+        # In Linux wildcard expansion done by bash so, add file to list
         if os.path.isfile(root):
             EXISTING_FILES.append(root)
         # Windows (& Linux with backslash prefix) does not expand wildcard '*'
@@ -654,6 +696,7 @@ def writeElement(elt, codec):
         if config.PYTHON_VERSION.startswith(config.PYTHON_3):
             codec.write(xmlOut)
 
+
 def checkFiles(recurse, wrap, paths):
     """Check the input argument path(s) for existing files and analyse them."""
     # Schema location (legacy, current)
@@ -758,6 +801,11 @@ def parseCommandLine():
                         default=False,
                         help="when analysing a directory, recurse into subdirectories \
                                 (implies --wrapper)")
+    PARSER.add_argument('--packetmarkers', '-p',
+                        action="store_true",
+                        dest="reportPacketMarkersFlag",
+                        default=False,
+                        help="Report packet-level codestream markers (plm, ppm, plt, ppt)")
     PARSER.add_argument('--verbose',
                         action="store_true",
                         dest="outputVerboseFlag",
@@ -801,6 +849,7 @@ def main():
     # Makes user-specified flags available to any module that imports 'config.py'
     # (here: 'boxvalidator.py')
     config.OUTPUT_VERBOSE_FLAG = args.outputVerboseFlag
+    config.OUTPUT_PACKET_MARKERS_FLAG = args.reportPacketMarkersFlag
     config.EXTRACT_NULL_TERMINATED_XML_FLAG = args.extractNullTerminatedXMLFlag
     config.INPUT_RECURSIVE_FLAG = args.inputRecursiveFlag
     config.INPUT_WRAPPER_FLAG = args.inputWrapperFlag
@@ -809,6 +858,18 @@ def main():
     config.LEGACY_XML_FLAG = args.legacyXMLFlag
     config.MIX_FLAG = args.mixFlag
 
+    # Report warning message for deprecated options
+    if config.INPUT_WRAPPER_FLAG:
+        msg = "--wrapper option is deprecated and will be removed in jplyzer 2.2"
+        shared.printWarning(msg)
+    if config.LEGACY_XML_FLAG:
+        msg = "--legacyout option is deprecated and will be removed in jplyzer 2.2"
+        shared.printWarning(msg)
+    # Report warning if Python version is 2.x
+    if config.PYTHON_VERSION.startswith(config.PYTHON_2):
+        msg = "support for Python 2 will be removed in jplyzer 2.2"
+        shared.printWarning(msg)
+
     # Exit if validation format is unknown
     if config.VALIDATION_FORMAT not in ['jp2', 'j2c']:
         msg = "'" + config.VALIDATION_FORMAT + "'  is not a supported value for --format"
